@@ -6,7 +6,7 @@ use strict;
 
 use vars qw/$VERSION/;
 
-$VERSION = '0.46';
+$VERSION = '0.48';
 
 # Package to store unsigned big integers in decimal and do math with them
 
@@ -573,35 +573,6 @@ sub _div_use_mul
     # now calculate $x / $yorg
     if (length(int($yorg->[-1])) == length(int($x->[-1])))
       {
-
-      # We take a shortcut here, because the result must be 
-      # between 1 and MAX_VAL (e.g. one element) and rem is not wanted.
-      if (!wantarray)
-        {
-        # fit's into one Perl scalar, so result can be computed directly
-        # cannot use int() here, because it rounds wrongly on some systems
-        #$x->[0] = int($x->[-1] / $yorg->[-1]);
-
- 	# Due to chopping up the number into parts, the two first parts
-	# may have only one or two digits. So we use more from the second
-	# parts (it always has at least two parts) for more accuracy:
-        # Round to 8 digits, then truncate result to integer:
-	my $x0 = $x->[-1];
-	my $y0 = $yorg->[-1];
-	if (length ($x0) < $BASE_LEN)		# len($x0) == len($y0)!
-	  {
-          $x0 .= substr('0' x $BASE_LEN . $x->[-2], -$BASE_LEN, $BASE_LEN);
-          $x0 = substr($x0,0,$BASE_LEN);
-          $y0 .= substr('0' x $BASE_LEN . $yorg->[-2], -$BASE_LEN, $BASE_LEN);
-          $y0 = substr($y0,0,$BASE_LEN);
-	  }
-        $x->[0] = int ( sprintf ("%.8f", $x0 / $y0 ) );
-
-        splice(@$x,1);			# keep single element
-        return $x;
-        }
- 
-      # wantarray: return (x,rem)
       # same length, so make full compare
 
       my $a = 0; my $j = scalar @$x - 1;
@@ -619,7 +590,8 @@ sub _div_use_mul
         splice(@$x,1);                  # keep single element
         $x->[0] = 0;                    # if $a < 0
         $x->[0] = 1 if $a == 0;         # $x == $y
-        return ($x,$rem);
+        return ($x,$rem) if wantarray;
+        return $x;
         }
       # $x >= $y, so proceed normally
       }
@@ -788,35 +760,6 @@ sub _div_use_div
 
     if (length(int($yorg->[-1])) == length(int($x->[-1])))
       {
-
-      # We take a shortcut here, because the result must be 
-      # between 1 and MAX_VAL (e.g. one element) and rem is not wanted.
-      if (!wantarray)
-        {
-        # fit's into one Perl scalar, so result can be computed directly
-        # cannot use int() here, because it rounds wrongly on some systems
-        #$x->[0] = int($x->[-1] / $yorg->[-1]);
-
- 	# Due to chopping up the number into parts, the two first parts
-	# may have only one or two digits. So we use more from the second
-	# parts (it always has at least two parts) for more accuracy:
-        # Round to 8 digits, then truncate result to integer:
-	my $x0 = $x->[-1];
-	my $y0 = $yorg->[-1];
-	if (length ($x0) < $BASE_LEN)		# len($x0) == len($y0)!
-	  {
-          $x0 .= substr('0' x $BASE_LEN . $x->[-2], -$BASE_LEN, $BASE_LEN);
-          $x0 = substr($x0,0,$BASE_LEN);
-          $y0 .= substr('0' x $BASE_LEN . $yorg->[-2], -$BASE_LEN, $BASE_LEN);
-          $y0 = substr($y0,0,$BASE_LEN);
-	  }
-        $x->[0] = int ( sprintf ("%.8f", $x0 / $y0 ) );
-
-        splice(@$x,1);			# keep single element
-        return $x;
-        }
- 
-      # wantarray: return (x,rem)
       # same length, so make full compare
 
       my $a = 0; my $j = scalar @$x - 1;
@@ -834,7 +777,8 @@ sub _div_use_div
         splice(@$x,1);			# keep single element
         $x->[0] = 0;			# if $a < 0
         $x->[0] = 1 if $a == 0; 	# $x == $y
-        return ($x,$rem);
+        return ($x,$rem) if wantarray;	# including remainder?
+        return $x;
         }
       # $x >= $y, so proceed normally
 
@@ -1796,7 +1740,7 @@ sub _as_hex
   while (@$x1 != 1 || $x1->[0] != 0)		# _is_zero()
     {
     ($x1, $xr) = _div($c,$x1,$x10000);
-    $es .= unpack($h,pack('v',$xr->[0]));	# XXX TODO: why pack('v',...)?
+    $es .= unpack($h,pack('V',$xr->[0]));
     }
   $es = reverse $es;
   $es =~ s/^[0]+//;   # strip leading zeros
@@ -1834,14 +1778,62 @@ sub _as_bin
   while (!(@$x1 == 1 && $x1->[0] == 0))		# _is_zero()
     {
     ($x1, $xr) = _div($c,$x1,$x10000);
-    $es .= unpack($b,pack('v',$xr->[0]));	# XXX TODO: why pack('v',...)?
-    # $es .= unpack($b,$xr->[0]);
+    $es .= unpack($b,pack('v',$xr->[0]));
     }
   $es = reverse $es;
   $es =~ s/^[0]+//;   # strip leading zeros
   '0b' . $es;					# return result prepended with 0b
   }
 
+sub _as_oct
+  {
+  # convert a decimal number to octal (ref to array, return ref to string)
+  my ($c,$x) = @_;
+
+  # fit's into one element (handle also 0 case)
+  return sprintf("0%o",$x->[0]) if @$x == 1;
+
+  my $x1 = _copy($c,$x);
+
+  my $es = '';
+  my $xr;
+  my $x1000 = [ 0100000 ];
+  while (@$x1 != 1 || $x1->[0] != 0)		# _is_zero()
+    {
+    ($x1, $xr) = _div($c,$x1,$x1000);
+    $es .= reverse sprintf("%05o", $xr->[0]);
+    }
+  $es = reverse $es;
+  $es =~ s/^[0]+//;   # strip leading zeros
+  '0' . $es;					# return result prepended with 0
+  }
+
+sub _from_oct
+  {
+  # convert a octal number to decimal (ref to string, return ref to array)
+  my ($c,$os) = @_;
+
+  # for older Perls, play safe
+  my $m = [ 0100000 ];
+  my $d = 5;					# 5 digits at a time
+
+  my $mul = _one();
+  my $x = _zero();
+
+  my $len = int( (length($os)-1)/$d );		# $d digit parts, w/o the '0'
+  my $val; my $i = -$d;
+  while ($len >= 0)
+    {
+    $val = substr($os,$i,$d);			# get oct digits
+    $val = oct($val);
+    $i -= $d; $len --;
+    my $adder = [ $val ];
+    _add ($c, $x, _mul ($c, $adder, $mul ) ) if $val != 0;
+    _mul ($c, $mul, $m ) if $len >= 0; 		# skip last mul
+    }
+  $x;
+  }
+
 sub _from_hex
   {
   # convert a hex number to decimal (ref to string, return ref to array)
@@ -1864,7 +1856,7 @@ sub _from_hex
   while ($len >= 0)
     {
     $val = substr($hs,$i,$d);			# get hex digits
-    $val =~ s/^[+-]?0x// if $len == 0;		# for last part only because
+    $val =~ s/^0x// if $len == 0;		# for last part only because
     $val = hex($val);				# hex does not like wrong chars
     $i -= $d; $len --;
     my $adder = [ $val ];
@@ -2064,6 +2056,7 @@ Math::BigInt v1.70 or later:
 
 	_from_hex(str)	return ref to new object from ref to hexadecimal string
 	_from_bin(str)	return ref to new object from ref to binary string
+	_from_oct(str)	return ref to new object from ref to octal string
 	
 	_as_hex(str)	return string containing the value as
 			unsigned hex string, with the '0x' prepended.
@@ -2148,7 +2141,7 @@ Original math code by Mark Biggar, rewritten by Tels L<http://bloodgate.com/>
 in late 2000.
 Seperated from BigInt and shaped API with the help of John Peacock.
 
-Fixed, speed-up, streamlined and enhanced by Tels 2001 - 2005.
+Fixed, speed-up, streamlined and enhanced by Tels 2001 - 2007.
 
 =head1 SEE ALSO
 
